import enum


class ChoicesMeta(enum.EnumMeta):
    """A metaclass for creating a enum choices."""

    def __contains__(cls, member):
        if not isinstance(member, enum.Enum):
            # Allow non-enums to match against member values.
            return any(x.value == member for x in cls)
        return super().__contains__(member)


class Choices(enum.Enum, metaclass=ChoicesMeta):
    """Class for creating enumerated choices."""

    def __str__(self):
        """
        Use value when cast to str, so that Choices set as model instance
        attributes are rendered as expected in templates and similar contexts.
        """
        return str(self.value)

    def __eq__(self, other):
        """Overrides the default implementation"""
        return str(self.value) == str(other)



import requests
import logging
import re


def get_codemagic_url(relative_url=None):
    url = 'https://api.codemagic.io'
    if relative_url:
        url = '{}/{}'.format(url, relative_url.lstrip('/'))
    return url


def prepare_headers(token) -> dict:
    return {
        'x-auth-token': token,
        'Content-Type': 'application/json',
    }


def send_get_request(url, data={}, headers={}) -> dict:
    r = requests.get(
        url,
        params=data,
        headers=headers,
    )
    return _handle_response(r)


def send_post_request(url, data={}, headers={}) -> dict:
    r = requests.post(
        url,
        headers=headers,
        json=data,
    )
    return _handle_response(r)


def _handle_response(response):
    if not (200 <= response.status_code < 300):
        logging.exception("Request failed", stack_info=True, )
    data = response.json()
    return data


def camel_to_snake(name):
    name = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', name).lower()



from datetime import datetime

from codemagic.apps import Application
from codemagic.enums import Choices


class BuildStatus(Choices):
    PREPARING = 'preparing'
    FINISHED = 'finished'
    BUILDING = 'building'
    FAILED = 'failed'
    CANCELED = 'canceled'
    QUEUED = 'queued'
    SKIPPED = 'skipped'
    NOT_FOUND = 'not_found'


class Build:
    pk: str = None
    app: Application = None
    status: BuildStatus = BuildStatus.NOT_FOUND
    workflow_id: str = None
    started_at: datetime = None

    def __init__(self):
        pass



from .builds import Build
from .utils import send_get_request, send_post_request, get_codemagic_url, prepare_headers, camel_to_snake


class Codemagic:

    def __init__(self, token):
        self.token = token

    def start_build(self, app_id: str, workflow_id: str, branch: str, environment: dict) -> Build:
        data = {
            'appId': app_id,
            'workflowId': workflow_id,
            'branch': branch,
            'environment': environment,
        }
        response = send_post_request(get_codemagic_url('/builds'), data=data, headers=prepare_headers(self.token))
        return self.get_build(response.get("buildId"))

    def list_of_builds(self, app_id=None, workflow_id=None, branch=None, tag=None) -> [Build]:
        data = {
            'appId': app_id,
            'workflowId': workflow_id,
            'branch': branch,
            'tag': tag
        }
        response = send_get_request(get_codemagic_url('/builds'), data=data, headers=prepare_headers(self.token))
        items = []
        for build_dict in response.get('builds', []):
            items.append(self.to_build(build_dict))
        return items

    def get_build(self, pk: str):
        response = send_get_request(get_codemagic_url(f'/builds/{pk}'), headers=prepare_headers(self.token))
        return self.to_build(response.get('build', {}))

    def cancel_build(self, pk: str):
        response = send_post_request(get_codemagic_url(f'/builds/{pk}/cancel'), headers=prepare_headers(self.token))
        return self.to_build(response.get('build', {}))

    @staticmethod
    def to_build(build_dict) -> Build:
        build = Build()
        keys = {}
        for key in build_dict.keys():
            keys[key] = camel_to_snake(key)

        keys['_id'] = 'pk'
        for key in keys:
            if hasattr(build, keys[key]) and build_dict.get(key, None):
                setattr(build, keys[key], build_dict[key])
        return build



from datetime import datetime

from codemagic.enums import Choices


class ProjectType(Choices):
    FLUTTER_APP = 'flutter-app'


class Application:
    pk: str
    app_name: str
    archived: bool
    branches: str
    created_at: datetime
    file_workflow_ids: list = []
    last_build_id: str
    project_type: ProjectType
